"""
Some code for testing and also as a usage demo
See bioweb.me/emil for benchmark experiments
"""

from numpy import r_
from numpy.random import seed
from matplotlib.pyplot import show
from optwok.mil.mi_data import EllipsoidData, MIMeanData
from optwok.mil.utility import create_perm, permfile_name
from optwok.mil.mi_classifier import MeanSVM
from optwok.mil.mi_classifier_emil import eMILDcaSocp, eMILDcaBfgs
from examples.emil_synthetic_ellipsoids import random_ellipsoids
from examples.emil_plotting import plot_ellipsoids, plot_hyperplane

def example_ellipsoid_2d():
    """Create some random ellipsoids, train and plot the resulting classifier"""
    reg_param = 1.2
    num_pos = 16
    num_neg = 10
    num_ex = num_pos + num_neg
    num_feat = 2
    
    seed(1)

    # Create some data
    perm_desc = 'example'
    perm_dataset = 'synthetic_ellipsoid_2d'
    create_perm('.', perm_dataset, perm_desc, 10, num_ex)
    means, covariances, labels = random_ellipsoids(num_neg=num_neg, num_pos=num_pos,
                                                   n=num_feat, mean_radius=7)
    data = EllipsoidData(means, covariances, labels)
    meandata = MIMeanData(means, labels)
    
    train_data, test_data = data.get_train_pred_data(permfile=permfile_name('.', perm_dataset, perm_desc), perm_idx=1,
                                                     model_selection=False, frac_ms=0.5, fold_cv=None, num_cv=None)

    print('Training mean SVM for initialization')
    meanclassifier = MeanSVM()
    mean_train_data, _ = meandata.get_train_pred_data(permfile=permfile_name('.', perm_dataset, perm_desc), perm_idx=1,
                                                     model_selection=False, frac_ms=0.5, fold_cv=None, num_cv=None)
    meanclassifier.train(mean_train_data, reg_param)

    classifier = eMILDcaBfgs()
    print('Training eMIL')
    classifier.train(train_data, reg_param, huber_alpha=0.1, v0=r_[meanclassifier.w, meanclassifier.b])
    
    print('Predicting')
    pred = classifier.predict(test_data)
    print(pred)
    print(test_data.labels)
    

    # Plot training
    plot_ellipsoids(train_data.means, train_data.covariances, train_data.labels, color=True)
    plot_hyperplane(classifier.w, classifier.b)
    show()

if __name__ == '__main__':
    example_ellipsoid_2d()
